/*
 *  typespeed - measures your typing speed
 *  Copyright (C) 1999-2003   Jani Ollikainen  <bestis@iki.fi>
 *                          & Jaakko Manelius  <jman@iki.fi>
 *  Copyright (C) 2006-2007   Tobias Stoeckmann  <tobias@bugol.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * menu.c - menu & misc drawing functions
 */

#include <sys/param.h>

#include <curses.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include "typespeed.h"
#include "version.h"

extern int		chooseruleset(void);
extern int		choosewordfile(int);
extern void		closenetwork(void);
extern int		initnetwork(char *, int);
extern unsigned short	level(int);
extern void		liima_mvgetnstr(int, int, char *, int);
extern void		multipmenu(void);
extern void		xcolor_set(short);

static int	 dochange(int);
void		 drawmenu(void);
void		 drawscreen(void);
void		 drawstatus(unsigned);
static int	 getnum(char *, long, long);
void		 multipmenu(void);
void		 nowordlist(void);
void		 optionmenu(void);
void		 pressanykey(int, int);
void		 rulesmenu(void);
void		 showhighscores(void);
void		 tellstory(void);
int		 wanttoreset(void);
int		 wordlistmenu(int, struct finfo *, char *);

extern int	 misses;
extern char	*rankki[11];
extern int	 typesocket;

/* Switches "number" from 1 to 0 or any other value to 1. */
static int
dochange(int number)
{
	return (number == 1) ? 0 : 1;
}

/* Draws main menu onto screen. */
void
drawmenu(void)
{
	drawscreen();

	mvprintw( 3, 30, "Typespeed %s", TVERSION);

	mvaddstr( 5, 30, "1. Test Your Speed");
	mvaddstr( 6, 30, "2. Network Head2Head");
	mvaddstr( 7, 30, "3. Story/Credits/RTFM!");
	mvaddstr( 8, 30, "4. Show Highscores");
	mvaddstr( 9, 30, "5. Options");
	mvaddstr(10, 30, "6. Game Rules");
	mvaddstr(11, 30, "7. Quit");

	mvaddstr(13, 30, "Choose: ");
}

/* Clears screen and draws status bar. */
void
drawscreen(void)
{
	clear();

	xcolor_set(4);
	move(22, 0);
	hline(ACS_HLINE, 80);
	mvaddch(23, 1, '>');
	mvaddch(23, 21, '<');

	/* sets color to 2: CLI, SER, CHE need this */
	drawstatus(0);

	if (opt.client)
		mvaddstr(22, 70, "CLI");
	else if (opt.server)
		mvaddstr(22, 70, "SER");
	if (opt.cheat)
		mvaddstr(22, 73, "CHE");

	xcolor_set(5);
	mvaddstr(23, 23, "Rank:");
	mvaddstr(23, 38, "Score:");
	mvaddstr(23, 49, "tCPS:");
	mvaddstr(23, 60, "cCPS:");
	mvaddstr(23, 71, "Misses:");
}

/*
 * Prints status values - no descriptions though!
 * Cursor will be put into "inputpos" (y = 21, x = 2 + inputpos).
 */
void
drawstatus(unsigned inputpos)
{
	mvprintw(23, 29, "%-9s", rankki[level(now.score)]);

	xcolor_set(3);
	mvprintw(23, 45, "%d", now.score);
	mvprintw(23, 55, "%2.2f", now.totalspeed);
	mvprintw(23, 66, "%2.2f", now.speed);
	mvprintw(23, 78, "%2d", misses);

	xcolor_set(2);
	move(23, 2 + inputpos);
}

/*
 * Asks player about a specific value named by "string". If the entered
 * value is smaller or equal min, or larger or equal max, 0 is returned.
 * If a string (and not a number) has been entered, 0 is returned, too.
 *
 * Sets rules.hightype to 0, so a customized rule set cannot reach top10.
 */
static int
getnum(char *string, long min, long max)
{
	long res;
	char *p;
	char entr[11];

	mvaddstr(17, 30, string);
	liima_mvgetnstr(17, 31 + strlen(string), entr, sizeof(entr) - 1);
	entr[sizeof(entr) - 1] = '\0';

	res = strtol(entr, &p, 10);
	if (p != NULL && *p != '\0')
		return 0;

	if (res > min && res < max) {
		rules.hightype = 0;
		return res;
	}
	return 0;
}

/* Draws multiplayer menu. */
void
multipmenu(void)
{
	int exitnow, wheretogo;
	char port[6], serv[MAXHOSTNAMELEN];

	exitnow = 0;
	serv[0] = '\0';

	do {
		drawscreen();
		mvprintw(3, 30, "Typespeed %s", TVERSION);

		mvaddstr(5, 30, "1. Server");
		mvaddstr(6, 30, "2. Client");
		mvaddstr(7, 30, "3. Port");
		mvaddstr(8, 30, "4. Return");

		xcolor_set(3);
		if (opt.server)
			mvaddstr(5, 44, "ON");
		if (opt.client)
			mvaddstr(6, 44, "ON");

		xcolor_set(6);
		if (!opt.server)
			mvaddstr(5, 44, "OFF");
		if (!opt.client)
			mvaddstr(6, 44, "OFF");

		xcolor_set(5);
		mvprintw(7, 44, "%d", opt.port);
		mvaddstr(10, 30, "Choose: ");

		nodelay(stdscr, 0);
		cbreak();

		while ((wheretogo = getch()) == ERR)
			;

		switch (wheretogo) {
		case '1':
			if (opt.client)
				break;
			if (opt.server) {
				closenetwork();
				opt.server = 0;
				typesocket = 0;
				exitnow = 0;
			} else {
				opt.server = 1;
				exitnow = 1;
			}
			break;
		case '2':
			if (opt.server)
				break;
			if (opt.client) {
				opt.client = 0;
				typesocket = 0;
				exitnow = 0;
			} else {
				opt.client = 1;
				mvaddstr(10, 30, "Enter Host:");
				liima_mvgetnstr(10, 42, serv,
				    sizeof(serv) - 1);
				serv[sizeof(serv) - 1] = '\0';
				exitnow = 1;
			}
			break;
		case '3':
			mvaddstr(10, 30, "Enter Port:");
			liima_mvgetnstr(10, 42, port, sizeof(port) - 1);
			port[sizeof(port) - 1] = '\0';
			opt.port = strtol(port, NULL, 10);
			if (opt.port <= 1024)
				opt.port = 6025;
			exitnow = 0;
			break;
		case '4':
			return;
			/* NOTREACHED */
			break;
		default:
			break;
		}
	} while (!exitnow);

	if (!opt.client && !opt.server)
		return;

	drawscreen();
	mvprintw(3, 30, "Typespeed %s", TVERSION);

	mvaddstr(5, 30, "When you get connect, choose a word list and");
	mvaddstr(6, 30, "then wait for the game start. The game starts as");
	mvaddstr(7, 30, "soon as the other player has chosen a word list.");
	refresh();

	while ((typesocket = initnetwork(serv, 1)) < 0)
		;

	flushinp();
}

/*
 * Prints "No word lists found...".
 * This functions just exists to split all output out of file.c.
 */
void
nowordlist(void)
{
	drawscreen();
	mvaddstr(1, 5, "No word lists found...");
	pressanykey(3, 5);
}

/* Draws option menu. */
void
optionmenu(void)
{
	int wheretogo;

	do {
		drawscreen();

		mvprintw(3, 30, "Typespeed %s", TVERSION);

		mvaddstr(5, 30, "1. Colors");
		mvaddstr(6, 30, "2. Cheat");
		mvaddstr(7, 30, "3. Return");

		xcolor_set(3);
		if (opt.usecolors)
			mvaddstr(5, 44, "ON");
		if (opt.cheat)
			mvaddstr(6, 44, "ON");

		xcolor_set(6);
		if (!opt.usecolors)
			mvaddstr(5, 44, "OFF");
		if (!opt.cheat)
			mvaddstr(6, 44, "OFF");

		xcolor_set(5);
		mvaddstr(12, 30, "Choose: ");

		nodelay(stdscr, 0);
		cbreak();

		while ((wheretogo = getch()) == ERR)
			;
		switch (wheretogo) {
		case '2':
			opt.cheat = dochange(opt.cheat);
			break;
		case '1':
			if (opt.usecolors)
				xcolor_set(2);
			opt.usecolors = dochange(opt.usecolors);
			break;
		default:
			break;
		}
	} while (wheretogo != '3');
}

/* This function waits until a key has been pressed */
void
pressanykey(int y, int x)
{
	xcolor_set(4);
	mvaddstr(y, x, "Press any key to continue...");
	while (getch() == ERR)
		;
}

/* Draws rules menu. */
void
rulesmenu(void)
{
	int wheretogo;
	long tmp;

	do {
		drawscreen();

		mvprintw( 3, 30, "Typespeed %s", TVERSION);

		mvaddstr( 5, 30, "Current Rule Set:");

		mvaddstr( 7, 30, "1. Load Another Rule Set");
		mvaddstr( 8, 30, "2. Misses Limit:");
		mvaddstr( 9, 30, "3. Word Length:    -");
		mvaddstr(10, 30, "4. Words On Screen:    -");
		mvaddstr(11, 30, "   High Score Enabled:");
		mvaddstr(12, 30, "6. Speed Step:");
		mvaddstr(13, 30, "7. Speed Range:    -");
		mvaddstr(14, 30, "8. Smoothness:");
		mvaddstr(15, 30, "9. Return");

		xcolor_set(3);
		mvprintw( 5, 48, "%s", rules.name);
		mvprintw( 8, 47, "%d", rules.misses);
		mvprintw( 9, 46, "%2d", rules.minlen);
		mvprintw( 9, 51, "%2d", rules.maxlen);
		mvprintw(10, 50, "%2d", rules.minwords);
		mvprintw(10, 55, "%2d", rules.maxwords);
		if (rules.hightype)
			mvaddstr(11, 53, "yes");
		else
			mvaddstr(11, 53, "no");
		mvprintw(12, 45, "%3d", rules.step);
		mvprintw(13, 46, "%2d", rules.minspeed);
		if (rules.maxspeed)
			mvprintw(13, 51, "%2d", rules.maxspeed);
		else
			mvaddstr(13, 51, "0 (unlimited)");
		if (rules.smooth)
			mvaddstr(14, 45, "yes");
		else
			mvaddstr(14, 45, "no");
		xcolor_set(5);
		mvaddstr(17, 30, "Choose: ");

		nodelay(stdscr, 0);
		cbreak();

		while ((wheretogo = getch()) == ERR)
			;
		switch (wheretogo) {
		case '1':
			chooseruleset();
			break;
		case '2':
			if ((tmp = getnum("Misses Limit:", 0, 100)))
				rules.misses = tmp;
			break;
		case '3':
			if ((tmp = getnum("Min Length:", 0, 20)))
				rules.minlen = tmp;
			if ((tmp = getnum("Max Length:", 0, 20)))
				rules.maxlen = tmp;
			if (rules.minlen > rules.maxlen)
				rules.maxlen = rules.minlen;
			break;
		case '4':
			if ((tmp = getnum("Min Words:", 0, 23)))
				rules.minwords = tmp;
			if ((tmp = getnum("Max Words:", 0, 23)))
				rules.maxwords = tmp;
			if (rules.minwords > rules.maxwords)
				rules.maxwords = rules.minwords;
			break;
		case '6':
			if ((tmp = getnum("Speed Step:", 0, LONG_MAX)))
				rules.step = tmp;
			break;
		case '7':
			if ((tmp = getnum("Min Speed:", 0, LONG_MAX)))
				rules.minspeed = tmp;
			if ((tmp = getnum("Max Speed:", 0, LONG_MAX)))
				rules.maxspeed = tmp;
			if (rules.minspeed > rules.maxspeed)
				rules.maxspeed = 0;
			break;
		case '8':
			rules.smooth = dochange(rules.smooth);
			rules.hightype = 0;
			break;
		default:
			break;
		}
	} while (wheretogo != '9');
}

/* Shows high scores. */
void
showhighscores(void)
{
	choosewordfile(1);
}

/* Draws credits onto screen. */
void
tellstory(void)
{
	drawscreen();

	mvprintw( 3, 2,
	    "Typespeed %s", TVERSION);
	mvaddstr( 5, 2,
	    "Typespeed is made by  Jani Ollikainen  <bestis@iki.fi>");
	mvaddstr( 6, 2,
	    "                    & Jaakko Manelius  <jman@iki.fi>");
	mvaddstr( 7, 2,
	    "Current maintainer is Tobias Stoeckmann  <tobias@bugol.de>");

	mvaddstr( 9, 2,
	    "Typespeed's idea is ripped from ztspeed (a dos game made");
	mvaddstr(10, 2,
	    "by Zorlim). Typespeed doesn't give scores that can be");
	mvaddstr(11, 2,
	    "compared with ztspeed anymore. We wanted to use our way.");
	mvaddstr(13, 2,
	    "Idea of the game should be clear to anyone, just type and");
	mvaddstr(14, 2,
	    "type it fast, or be a lewser.");
	mvaddstr(16, 2,
	    "Bugs/Ideas/Comments to <tobias@bugol.de>");

	nodelay(stdscr, 0);
	cbreak();
	flushinp();

	pressanykey(18, 2);
}

/*
 * This function asks player if he wants to reset a high score file.
 * This functions just exists to split all output out of file.c.
 */
int
wanttoreset(void)
{
	int ch;

	xcolor_set(4);
	mvaddstr(16, 3, "Do you want to RESET this [y/n]? ");
	do {
		while ((ch = getch()) == ERR)
			;
		ch = tolower(ch);
	} while ((ch != 'y') && (ch != 'n'));

	if (ch == 'y')
		return 1;
	return 0;
}

/* Prints word list menu. */
int
wordlistmenu(int tot, struct finfo *thingie, char *title)
{
	int a, ch, cpos, k;

	drawscreen();
	xcolor_set(4);
	mvprintw(2, 5, "Choose a %s (UP/DOWN/ENTER):", title);
	if (tot > 17)
		for (k = 0; k < 17; k++)
			mvaddstr(k + 4, 5, thingie[k].descr);
	else
		for (k = 0; k < tot; k++)
			mvaddstr(k + 4, 5, thingie[k].descr);

	cpos = k = 0;
	mvaddstr(4, 1, "->");

	for (a = 1; a;) {
		/*
		 * And thanks for xmunkki, he made my day and
		 * "solved" this, what i didn't get right (or did
		 * but i used 2 extra variables. :)
		 */
		while ((ch = getch()) == ERR)
			;
		switch (ch) {
		case KEY_UP:
			if (cpos > 0) {
				cpos--;
				if (k > 0)
					k--;
			} else if (k > 0) {
				k--;
				move(20, 5);
				deleteln();
				move(4, 5);
				insertln();
				mvaddstr(4, 5, thingie[k].descr);
			}
			break;
		case KEY_DOWN:
			if ((cpos < 16) && (cpos < tot - 1)) {
				cpos++;
				if (k < tot - 1)
					k++;
			} else if (k < tot - 1) {
				k++;
				move(4, 5);
				deleteln();
				move(20, 5);
				insertln();
				mvaddstr(20, 5, thingie[k].descr);
			}
			break;
		case ' ':
		case '\n':
			a = 0;
			break;
		default:
			break;
		}
		mvaddstr(cpos + 3, 1, "  ");
		mvaddstr(cpos + 5, 1, "  ");
		mvaddstr(cpos + 4, 1, "->");
	}

	return k;
}

